@@ -2,10 +2,14 @@
 
 #' Python configuration
 #'
-#' Information on Python and Numpy versions detected
+#' Retrieve information about the version of Python currently being used by
+#' `reticulate`.
 #'
-#' @return Python configuration object; Logical indicating whether Python
-#'   bindings are available
+#' If Python has not yet been initialized, then calling `py_config()` will force
+#' the initialization of Python. See [py_discover_config()] for more details.
+#'
+#' @return Information about the version of Python in use, as an \R list with
+#'   class `"py_config"`.
 #'
 #' @export
 py_config <- function() {
@@ -13,6 +17,55 @@ py_config <- function() {
   .globals$py_config
 }
 
+#' Python version
+#'
+#' Get the version of Python currently being used by `reticulate`.
+#'
+#' @return The version of Python currently used, or `NULL` if Python has
+#'   not yet been initialized by `reticulate`.
+#'
+#' @export
+py_version <- function() {
+
+  if (!py_available(initialize = FALSE))
+    return(NULL)
+
+  config <- py_config()
+  numeric_version(config$version)
+
+}
+
+#' Python executable
+#'
+#' Get the path to the Python executable that `reticulate` has been configured
+#' to use. If Python has already been initialized, then `reticulate` will
+#' choose the currently-active copy of Python.
+#'
+#' This can occasionally be useful if you'd like to interact with Python (or its
+#' modules) via a subprocess; for example you might choose to install a package
+#' with `pip`:
+#'
+#' ```
+#' system2(py_exe(), c("-m", "pip", "install", "numpy"))
+#' ```
+#'
+#' and so you can also have greater control over how these modules are invoked.
+#'
+#' @return The path to the Python executable `reticulate` has been configured
+#'   to use.
+#'
+#' @export
+py_exe <- function() {
+
+  # if python has already been initialized, use that
+  if (!is.null(.globals$py_config))
+    return(.globals$py_config$python)
+
+  # otherwise, guess what version of python we'd use
+  py_discover_config()$python
+
+}
+
 
 #' Build Python configuration error message
 #'
@@ -43,31 +96,38 @@ py_config_error_message <- function(prefix) {
 #'
 #' @export
 py_available <- function(initialize = FALSE) {
+
   if (is_python_initialized())
+    return(.globals$py_config$available)
+
+  if (!initialize)
+    return(FALSE)
+
+  tryCatch({
+    ensure_python_initialized()
     .globals$py_config$available
-  else if (initialize) {
-    tryCatch({
-      ensure_python_initialized()
-      .globals$py_config$available
-    }, error = function(e) FALSE)
-  } else {
-    FALSE
-  }
+  }, error = function(e) FALSE)
+
 }
 
 
 #' @rdname py_available
 #' @export
 py_numpy_available <- function(initialize = FALSE) {
+
   if (!py_available(initialize = initialize))
-    FALSE
-  else
-    py_numpy_available_impl()
+    return(FALSE)
+
+  py_numpy_available_impl()
+
 }
 
 
 #' Check if a Python module is available on this system.
 #'
+#' Note that this function will also attempt to initialize Python
+#' before checking if the requested module is available.
+#'
 #' @param module The name of the module.
 #'
 #' @return `TRUE` if the module is available and can be loaded;
@@ -87,52 +147,91 @@ py_module_available <- function(module) {
 #'
 #' @param required_module A optional module name that must be available
 #'   in order for a version of Python to be used.
+#'
 #' @param use_environment An optional virtual/conda environment name
-#'   to prefer in the search
+#'   to prefer in the search.
 #'
 #' @return Python configuration object.
 #'
 #' @export
 py_discover_config <- function(required_module = NULL, use_environment = NULL) {
 
+  # check if python symbols can already be found in the current process
+  main_process_info <- main_process_python_info()
+  if (!is.null(main_process_info)) {
+    python_version <- normalize_python_path(main_process_info$python)$path
+    config <- python_config(python_version, required_module, forced = "the current process")
+    return(config)
+  }
+
   # if PYTHON_SESSION_INITIALIZED is specified then use it without scanning
   # further (this is a "hard" requirement because an embedding process may
   # set this to indicate that the python interpreter is already loaded)
   py_session_initialized <- py_session_initialized_binary()
   if (!is.null(py_session_initialized)) {
     python_version <- normalize_python_path(py_session_initialized)$path
-    config <- python_config(python_version, required_module, python_version, forced = "PYTHON_SESSION_INITIALIZED")
+    config <- python_config(python_version, required_module, forced = "PYTHON_SESSION_INITIALIZED")
     return(config)
   }
 
   # if RETICULATE_PYTHON is specified then use it without scanning further
   reticulate_env <- Sys.getenv("RETICULATE_PYTHON", unset = NA)
   if (!is.na(reticulate_env)) {
+
     python_version <- normalize_python_path(reticulate_env)
     if (!python_version$exists)
       stop("Python specified in RETICULATE_PYTHON (", reticulate_env, ") does not exist")
+
     python_version <- python_version$path
-    config <- python_config(python_version, required_module, python_version, forced = "RETICULATE_PYTHON")
+    config <- python_config(python_version, required_module, forced = "RETICULATE_PYTHON")
     return(config)
+
   }
 
   # if RETICULATE_PYTHON_ENV is specified then use that
   reticulate_python_env <- Sys.getenv("RETICULATE_PYTHON_ENV", unset = NA)
   if (!is.na(reticulate_python_env)) {
+
     python <- python_binary_path(reticulate_python_env)
     python_version <- normalize_python_path(python)
     if (!python_version$exists)
       stop("Python specified in RETICULATE_PYTHON_ENV (", reticulate_python_env, ") does not exist")
+
     path <- python_version$path
-    config <- python_config(path, required_module, path, forced = "RETICULATE_PYTHON_ENV")
+    config <- python_config(path, required_module, forced = "RETICULATE_PYTHON_ENV")
     return(config)
+
   }
 
-  # next look for a required python version (e.g. use_python("/usr/bin/python", required = TRUE))
+  # if we're working within a project that contains a pyproject.toml file,
+  # then use the copy of Python associated with the poetry environment
+  config <- tryCatch(poetry_config(required_module), error = identity)
+  if (!inherits(config, "error") && !is.null(config))
+    return(config)
+
+  # if we're working within a project that contains a Pipfile, then
+  # use the copy of Python associated with that pipenv
+  config <- tryCatch(pipenv_config(required_module), error = identity)
+  if (!inherits(config, "error") && !is.null(config))
+    return(config)
+
+  # next look for a required python version
+  # (e.g. use_python("/usr/bin/python", required = TRUE))
   required_version <- .globals$required_python_version
   if (!is.null(required_version)) {
     python_version <- normalize_python_path(required_version)$path
-    config <- python_config(python_version, required_module, python_version, forced = "use_python function")
+    config <- python_config(python_version, required_module, forced = "use_python function")
+    return(config)
+  }
+
+  # if RETICULATE_PYTHON_FALLBACK is specified then use it
+  reticulate_env <- Sys.getenv("RETICULATE_PYTHON_FALLBACK", unset = NA)
+  if (!is.na(reticulate_env)) {
+    python_version <- normalize_python_path(reticulate_env)
+    if (!python_version$exists)
+      stop("Python specified in RETICULATE_PYTHON_FALLBACK (", reticulate_env, ") does not exist")
+    python_version <- python_version$path
+    config <- python_config(python_version, required_module, python_version, forced = "RETICULATE_PYTHON_FALLBACK")
     return(config)
   }
 
@@ -144,7 +243,7 @@ py_discover_config <- function(required_module = NULL, use_environment = NULL) {
   python_virtualenvs <- python_virtualenv_versions()
   r_reticulate_python_envs <- python_virtualenvs[python_virtualenvs$name == "r-reticulate", ]
   python_versions <- c(python_versions, r_reticulate_python_envs$python)
-  
+
   # next look in virtual environments that have a required module derived name
   if (!is.null(required_module)) {
     # filter by required module
@@ -152,12 +251,12 @@ py_discover_config <- function(required_module = NULL, use_environment = NULL) {
     module_python_envs <- python_virtualenvs[python_virtualenvs$name %in% envnames, ]
     python_versions <- c(python_versions, module_python_envs$python)
   }
-  
+
   # look for conda environments
   python_condaenvs <- python_conda_versions()
   r_reticulate_python_envs <- python_condaenvs[python_condaenvs$name == "r-reticulate", ]
   python_versions <- c(python_versions, r_reticulate_python_envs$python)
-  
+
   # next look in conda envs that have a required module derived name
   if (!is.null(required_module)) {
     # filter by required module
@@ -165,81 +264,66 @@ py_discover_config <- function(required_module = NULL, use_environment = NULL) {
     module_python_envs <- python_condaenvs[python_condaenvs$name %in% envnames, ]
     python_versions <- c(python_versions, module_python_envs$python)
   }
-  
+
   # look for r-reticulate environment in miniconda
   # if the environment doesn't exist, and the user hasn't requested a separate
   # environment, then we'll prompt for installation of miniconda
   miniconda <- miniconda_conda()
   if (!file.exists(miniconda)) {
-    
+
     can_install_miniconda <-
       is_interactive() &&
       length(python_versions) == 0 &&
       miniconda_enabled() &&
       miniconda_installable()
-    
+
     if (can_install_miniconda)
       miniconda_install_prompt()
-    
+
   }
-  
+
   # if the earlier branch installed miniconda, it may exist now -- if so,
   # try to activate it
   if (file.exists(miniconda)) {
-    
+
     # create the conda environment if necessary
     envpath <- miniconda_python_envpath()
     if (!file.exists(envpath)) {
       python <- miniconda_python_package()
       conda_create(envpath, packages = c(python, "numpy"), conda = miniconda)
     }
-    
+
     # bind to it
     miniconda_python <- conda_python(envpath, conda = miniconda)
     config <- python_config(miniconda_python, NULL, miniconda_python)
     return(config)
-    
+
   }
-  
-  # the user might have opted out for miniconda but could still have a 
+
+  # the user might have opted out for miniconda but could still have a
   # conda isntallation. In this case, we should the r-reticulate env
   # we use the same python version as we would install with miniconda.
   if (conda_installed() && nrow(conda_list()) == 0) {
     python <- miniconda_python_package()
     conda_create("r-reticulate", packages = c(python, "numpy"), conda = conda_binary())
-    
+
     # gather python conda versions one again as they might exist now that
     # we created the environment
     python_condaenvs <- python_conda_versions()
     r_reticulate_python_envs <- python_condaenvs[python_condaenvs$name == "r-reticulate", ]
     python_versions <- c(python_versions, r_reticulate_python_envs$python)
   }
-  
+
   # join virtualenv, condaenv environments together
   python_envs <- rbind(python_virtualenvs, python_condaenvs)
-  
+
   # look on system path
   python <- as.character(Sys.which("python3"))
   if (nzchar(python))
     python_versions <- c(python_versions, python)
 
   # provide other common locations
-  if (is_windows()) {
-    python_versions <- c(python_versions, py_versions_windows()$executable_path)
-  } else {
-    python_versions <- c(python_versions,
-                         "/usr/bin/python3",
-                         "/usr/local/bin/python3",
-                         "/opt/python/bin/python3",
-                         "/opt/local/python/bin/python3",
-                         "/usr/bin/python",
-                         "/usr/local/bin/python",
-                         "/opt/python/bin/python",
-                         "/opt/local/python/bin/python",
-                         path.expand("~/anaconda3/bin/python"),
-                         path.expand("~/anaconda/bin/python")
-    )
-  }
+  python_versions <- c(python_versions, py_discover_config_fallbacks())
 
   # next add all known virtual environments
   python_versions <- c(python_versions, python_envs$python)
@@ -266,6 +350,23 @@ py_discover_config <- function(required_module = NULL, use_environment = NULL) {
     # get the config
     config <- python_config(python_version, required_module, python_versions)
 
+    # if this is a conda python installation, then create an r-reticulate
+    # environment and use that instead
+    initenv <-
+      identical(getOption("reticulate.conda.autoclone", FALSE), TRUE) &&
+      identical(getOption("reticulate.python.initializing"), TRUE) &&
+      identical(config$conda, TRUE)
+
+    if (initenv) {
+      fmt <- "* Found conda installation at %s; creating 'r-reticulate' environment ..."
+      messagef(fmt, pretty_path(config$prefix))
+      conda_clone("r-reticulate")
+      conda_install("r-reticulate", packages = "numpy")
+      python <- conda_python("r-reticulate")
+      config <- python_config(python, required_module, python_versions)
+      return(config)
+    }
+
     # if we have a required module ensure it's satisfied.
     # also check architecture (can be an issue on windows)
     has_python_gte_27 <- as.numeric_version(config$version) >= "2.7"
@@ -287,6 +388,46 @@ py_discover_config <- function(required_module = NULL, use_environment = NULL) {
     return(NULL)
 }
 
+py_discover_config_fallbacks <- function() {
+
+  # prefer conda python if available
+  conda <- find_conda()[[1L]]
+  if (!is.null(conda) && file.exists(conda)) {
+
+    pythons <- tryCatch(
+      conda_python(envname = "base", conda = conda, all = TRUE),
+      error = identity
+    )
+
+    if (is.character(pythons))
+      return(pythons)
+
+  }
+
+  # on Windows, try looking in the registry
+  if (is_windows())
+    return(py_versions_windows()$executable_path)
+
+  # otherwise, just search some default locations
+  prefixes <- c(
+    "/opt/local/python",
+    "/opt/python",
+    "/usr/local",
+    "/usr"
+  )
+
+  suffixes <- c("bin/python3", "bin/python")
+  grid <- expand.grid(
+    prefix = prefixes,
+    suffix = suffixes,
+    KEEP.OUT.ATTRS = FALSE,
+    stringsAsFactors = FALSE
+  )
+
+  paste(grid$prefix, grid$suffix, sep = "/")
+
+}
+
 
 #' Discover versions of Python installed on a Windows system
 #'
@@ -314,9 +455,9 @@ python_virtualenv_versions <- function() {
 }
 
 python_conda_versions <- function() {
-  
+
   if (is_windows()) {
-    
+
     # list all conda environments
     conda_envs <- data.frame(name = character(),
                              python = character(),
@@ -326,11 +467,11 @@ python_conda_versions <- function() {
     for (conda in file.path(anaconda_registry_versions$install_path, "Scripts", "conda.exe")) {
       conda_envs <- rbind(conda_envs, conda_list(conda = conda))
     }
-    
+
     conda_envs
-    
+
   } else {
-    
+
     env_dirs <- c("~/anaconda/envs",
                   "~/anaconda2/envs",
                   "~/anaconda3/envs",
@@ -352,13 +493,13 @@ python_conda_versions <- function() {
                   "~/opt/anaconda3/envs",
                   "~/opt/anaconda4/envs",
                   "~")
-    
+
     python_env_binaries <- python_environments(env_dirs)
     data.frame(name = basename(dirname(dirname(python_env_binaries))),
                python = python_env_binaries,
                stringsAsFactors = FALSE)
   }
-  
+
 }
 
 python_environments <- function(env_dirs, required_module = NULL) {
@@ -404,6 +545,39 @@ python_munge_path <- function(python) {
   # https://github.com/rstudio/reticulate/issues/367
   python_home <- dirname(python)
   python_dirs <- c(normalizePath(python_home))
+
+  # fix rpath for anaconda libmkl
+  if (is_osx()) {
+    libmkl <- file.path(python_home, "../lib/libmkl_intel_thread.dylib")
+    if (file.exists(libmkl)) {
+      libmkl <- normalizePath(libmkl)
+      args <- c("-add_rpath", shQuote(dirname(libmkl)), libmkl)
+      system2("install_name_tool", args, stdout = FALSE, stderr = FALSE)
+    }
+  }
+
+
+
+  if (is_conda_python(python)) {
+    conda_info <- get_python_conda_info(python)
+
+    if (numeric_conda_version(conda_info$conda) >= "4.9") {
+
+      new_path <- conda_run(
+        "python",
+        c("-c", shQuote("import os; print(os.environ['PATH'])")),
+        conda = conda_info$conda,
+        envname = conda_info$root,
+        stdout = TRUE
+      )
+
+      old_path <- Sys.getenv("PATH")
+      Sys.setenv("PATH" = new_path)
+      return(old_path)
+    }
+
+  }
+
   if (is_windows()) {
 
     # include the Scripts path, as well
@@ -422,32 +596,57 @@ python_munge_path <- function(python) {
     if (file.exists(python_library_bin))
       python_dirs <- c(python_dirs, normalizePath(python_library_bin))
   }
-  
-  # fix rpath for anaconda libmkl
-  if (is_osx()) {
-    libmkl <- file.path(python_home, "../lib/libmkl_intel_thread.dylib")
-    if (file.exists(libmkl)) {
-      libmkl <- normalizePath(libmkl)
-      args <- c("-add_rpath", shQuote(dirname(libmkl)), libmkl)
-      system2("install_name_tool", args, stdout = FALSE, stderr = FALSE)
-    }
-  }
+
 
   path_prepend(python_dirs)
 
 }
 
-python_config <- function(python, required_module, python_versions, forced = NULL) {
+python_config_impl <- function(python) {
 
+  script <- system.file("config/config.py", package = "reticulate")
+  config <- system2(
+    command = python,
+    args    = shQuote(script),
+    stdout  = TRUE,
+    stderr  = FALSE
+  )
+
+  # check for error
+  status <- attr(config, "status")
+  if (!is.null(status)) {
+    errmsg <- attr(config, "errmsg")
+    stop("Error ", status, " occurred running ", python, ": ", errmsg)
+  }
+
+  # on macOS, if we see some variables referencing /Applications/Xcode.app
+  # but that doesn't actually exist, then redirect to default CLT
+  if (is_osx()) {
+    clt <- "/Library/Developer/CommandLineTools"
+    xcode <- "/Applications/Xcode.app/Contents/Developer"
+    if (file.exists(clt))
+      config <- gsub(xcode, clt, config, fixed = TRUE)
+  }
+
+  # return config
+  config
+
+}
+
+python_config <- function(python,
+                          required_module = NULL,
+                          python_versions = python,
+                          forced = NULL)
+{
   # normalize and remove duplicates
   python <- canonical_path(python)
   python_versions <- canonical_path(python_versions)
   python_versions <- unique(python_versions)
-  
+
   # update and restore PATH when done
   oldpath <- python_munge_path(python)
   on.exit(Sys.setenv(PATH = oldpath), add = TRUE)
-  
+
   # set LD_LIBRARY_PATH on Linux as well, just to make sure Python libraries
   # can be resolved if necessary (also need to guard against users who munge
   # LD_LIBRARY_PATH in a way that breaks dynamic lookup of Python libraries)
@@ -473,41 +672,34 @@ python_config <- function(python, required_module, python_versions, forced = NUL
   }
 
   # execute config script
-  config_script <- system.file("config/config.py", package = "reticulate")
-  config <- system2(
-    command = python,
-    args    = shQuote(config_script),
-    stdout  = TRUE,
-    stderr  = FALSE
-  )
-  
-  # check for error
-  status <- attr(config, "status")
-  if (!is.null(status)) {
-    errmsg <- attr(config, "errmsg")
-    stop("Error ", status, " occurred running ", python, " ", errmsg)
-  }
+  config <- python_config_impl(python)
 
   # read output as dcf
   config_connection <- textConnection(config)
-  on.exit(close(config_connection))
+  on.exit(close(config_connection), add = TRUE)
   config <- read.dcf(config_connection, all = TRUE)
 
   # get the full textual version and the numeric version, check for anaconda
   version_string <- config$Version
   version <- config$VersionNumber
-  anaconda <- grepl("continuum", tolower(version_string)) || grepl("anaconda", tolower(version_string))
+  conda <- grepl("conda", version_string, ignore.case = TRUE)
+  anaconda <- grepl("anaconda|continuum", version_string, ignore.case = TRUE)
   architecture <- config$Architecture
 
-  # determine the location of libpython (see also # https://github.com/JuliaPy/PyCall.jl/blob/master/deps/build.jl)
-  if (is_windows()) {
-    
+  # determine the location of libpython
+  # see also: https://github.com/JuliaPy/PyCall.jl/blob/master/deps/build.jl
+  main_process_info <- main_process_python_info()
+  if (!is.null(main_process_info)) {
+    # either we have the main process libpython, or NA in case of PIE executable
+    libpython <- main_process_info$libpython
+  } else if (is_windows()) {
+
     # construct DLL name
     dll <- sprintf("python%s.dll", gsub(".", "", version, fixed = TRUE))
-    
+
     # default to just using dll as libpython path (implies lookup on PATH)
     libpython <- dll
-    
+
     # search for python DLL in one of the declared prefixes
     roots <- c(
       dirname(python),
@@ -515,7 +707,7 @@ python_config <- function(python, required_module, python_versions, forced = NUL
       config$ExecPrefix,
       config$BaseExecPrefix
     )
-    
+
     for (root in roots) {
       candidate <- file.path(root, dll)
       if (file.exists(candidate)) {
@@ -523,17 +715,9 @@ python_config <- function(python, required_module, python_versions, forced = NUL
         break
       }
     }
-    
+
   } else {
-    
-    # (note that the LIBRARY variable has the name of the static library)
-    python_libdir_config <- function(var) {
-      python_libdir <- config[[var]]
-      ext <- switch(Sys.info()[["sysname"]], Darwin = ".dylib", Windows = ".dll", ".so")
-      pattern <- paste0("^libpython", version, "d?m?", ext)
-      libpython <- list.files(python_libdir, pattern = pattern, full.names = TRUE)
-    }
-    
+
     # default to NULL
     libpython <- NULL
 
@@ -541,12 +725,12 @@ python_config <- function(python, required_module, python_versions, forced = NUL
     # (necessary for virtualenvs that don't copy over the shared library)
     libsrcs <- c("LIBPL", "LIBDIR", "Prefix", "ExecPrefix", "BaseExecPrefix")
     for (libsrc in libsrcs) {
-      
+
       # skip null entries in config
       src <- config[[libsrc]]
       if (is.null(src))
         next
-      
+
       # get appropriate libpython extension for platform
       ext <- switch(
         Sys.info()[["sysname"]],
@@ -554,7 +738,7 @@ python_config <- function(python, required_module, python_versions, forced = NUL
         Windows = ".dll",
         ".so"
       )
-      
+
       # try to resolve libpython in this location
       pattern <- sprintf("^libpython%sd?m?%s", version, ext)
       candidates <- list.files(src, pattern = pattern, full.names = TRUE)
@@ -562,7 +746,7 @@ python_config <- function(python, required_module, python_versions, forced = NUL
         libpython <- candidates
         break
       }
-      
+
     }
   }
 
@@ -622,6 +806,7 @@ python_config <- function(python, required_module, python_versions, forced = NUL
     version              = version,
     architecture         = architecture,
     anaconda             = anaconda,
+    conda                = conda,
     numpy                = numpy,
     required_module      = required_module,
     required_module_path = required_module_path,
@@ -629,30 +814,39 @@ python_config <- function(python, required_module, python_versions, forced = NUL
     python_versions      = python_versions,
     forced               = forced
   )
-  
+
   class(info) <- "py_config"
   info
-
 }
 
 #' @export
 str.py_config <- function(object, ...) {
-  x <- object
+  NextMethod()
+}
+
+#' @export
+format.py_config <- function(x, ...) {
+
   out <- ""
   out <- paste0(out, "python:         ", x$python, "\n")
-  out <- paste0(out, "libpython:      ", ifelse(is.null(x$libpython), "[NOT FOUND]", x$libpython), ifelse(is_windows() || is.null(x$libpython) || file.exists(x$libpython), "", "[NOT FOUND]"), "\n")
+  out <- paste0(out, "libpython:      ", ifelse(is.null(x$libpython), "[NOT FOUND]", x$libpython), ifelse(is_windows() || is.null(x$libpython) || is.na(x$libpython) || file.exists(x$libpython), "", "[NOT FOUND]"), "\n")
   out <- paste0(out, "pythonhome:     ", ifelse(is.null(x$pythonhome), "[NOT FOUND]", x$pythonhome), "\n")
+
   if (nzchar(x$virtualenv_activate))
     out <- paste0(out, "virtualenv:     ", x$virtualenv_activate, "\n")
+
   out <- paste0(out, "version:        ", x$version_string, "\n")
+
   if (is_windows())
     out <- paste0(out, "Architecture:   ", x$architecture, "\n")
+
   if (!is.null(x$numpy)) {
     out <- paste0(out, "numpy:          ", x$numpy$path, "\n")
     out <- paste0(out, "numpy_version:  ", as.character(x$numpy$version), "\n")
   } else {
     out <- paste0(out, "numpy:           [NOT FOUND]\n")
   }
+
   if (!is.null(x$required_module)) {
     out <- paste0(out, sprintf("%-16s", paste0(x$required_module, ":")))
     if (!is.null(x$required_module_path))
@@ -660,20 +854,23 @@ str.py_config <- function(object, ...) {
     else
       out <- paste0(out, "[NOT FOUND]\n")
   }
+
   if (!is.null(x$forced)) {
     out <- paste0(out, "\nNOTE: Python version was forced by ", x$forced, "\n")
   }
+
   if (length(x$python_versions) > 1) {
     out <- paste0(out, "\npython versions found: \n")
     python_versions <- paste0(" ", x$python_versions, collapse = "\n")
     out <- paste0(out, python_versions, sep = "\n")
   }
+
   out
 }
 
 #' @export
 print.py_config <- function(x, ...) {
-  cat(str(x))
+  cat(format(x))
 }
 
 
@@ -736,7 +933,7 @@ reticulate_python_versions <- function() {
         python_versions <- c(python_versions, python$path)
     }
   }
-  
+
   # return them
   python_versions
 }
@@ -821,7 +1018,7 @@ read_python_versions_from_registry <- function(hive, key,type=key) {
                 arch <- NA
               }
             } else { # type == "PythonCore"
-              matches <- regexec("^(\\d)\\.(\\d)(?:-(32|64))?$", version)
+              matches <- regexec("^(\\d+)\\.(\\d+)(?:-(32|64))?$", version)
               matches <- regmatches(version, matches)[[1]]
               if (length(matches) == 4) {
                 version <- paste(matches[[2]], matches[[3]], sep = ".")
@@ -941,5 +1138,3 @@ py_session_initialized_binary <- function() {
   # return
   python_binary
 }
-
-
