@@ -1,6 +1,6 @@
 
-#ifndef __LIBPYTHON_HPP__
-#define __LIBPYTHON_HPP__
+#ifndef RETICULATE_LIBPYTHON_H
+#define RETICULATE_LIBPYTHON_H
 
 #include <string>
 #include <ostream>
@@ -15,6 +15,7 @@
 #define _PYTHON_API_VERSION 1013
 #define _PYTHON3_ABI_VERSION 3
 
+namespace reticulate {
 namespace libpython {
 
 #if _WIN32 || _WIN64
@@ -115,8 +116,8 @@ typedef struct PyModuleDef{
 } PyModuleDef;
 
 typedef struct PyCompilerFlags{
-  int cf_flags;  
-  int cf_feature_version; 
+  int cf_flags;
+  int cf_feature_version;
 } PyCompilerFlags;
 
 typedef Py_ssize_t Py_hash_t;
@@ -124,6 +125,7 @@ typedef Py_ssize_t Py_hash_t;
 LIBPYTHON_EXTERN PyTypeObject* PyFunction_Type;
 LIBPYTHON_EXTERN PyTypeObject* PyModule_Type;
 LIBPYTHON_EXTERN PyTypeObject* PyType_Type;
+LIBPYTHON_EXTERN PyTypeObject* PyProperty_Type;
 
 LIBPYTHON_EXTERN PyObject* Py_None;
 LIBPYTHON_EXTERN PyObject* Py_Unicode;
@@ -161,9 +163,14 @@ void initialize_type_objects(bool python3);
 
 LIBPYTHON_EXTERN void (*Py_Initialize)();
 LIBPYTHON_EXTERN int (*Py_IsInitialized)();
+LIBPYTHON_EXTERN const char* (*Py_GetVersion)();
+LIBPYTHON_EXTERN char* (*Py_GetProgramFullPath_v2)();
+LIBPYTHON_EXTERN wchar_t* (*Py_GetProgramFullPath)();
+
 
 LIBPYTHON_EXTERN int (*Py_AddPendingCall)(int (*func)(void *), void *arg);
 LIBPYTHON_EXTERN void (*PyErr_SetInterrupt)();
+LIBPYTHON_EXTERN void (*PyErr_CheckSignals)();
 
 LIBPYTHON_EXTERN PyObject* (*Py_InitModule4)(const char *name, PyMethodDef *methods,
            const char *doc, PyObject *self,
@@ -182,6 +189,7 @@ LIBPYTHON_EXTERN PyObject* (*Py_BuildValue)(const char *format, ...);
 LIBPYTHON_EXTERN void (*Py_IncRef)(PyObject *);
 LIBPYTHON_EXTERN void (*Py_DecRef)(PyObject *);
 
+LIBPYTHON_EXTERN int (*PyObject_Print)(PyObject* o, FILE* fp, int flags);
 LIBPYTHON_EXTERN PyObject* (*PyObject_Str)(PyObject *);
 
 LIBPYTHON_EXTERN int (*PyObject_IsInstance)(PyObject *object, PyObject *typeorclass);
@@ -258,6 +266,9 @@ LIBPYTHON_EXTERN PyObject* (*PyUnicode_FromString)(const char *u);
 
 LIBPYTHON_EXTERN void (*PyErr_Clear)();
 LIBPYTHON_EXTERN void (*PyErr_Fetch)(PyObject **, PyObject **, PyObject **);
+LIBPYTHON_EXTERN void (*PyErr_Restore)(PyObject *, PyObject *, PyObject *);
+LIBPYTHON_EXTERN void (*PyErr_SetNone)(PyObject*);
+LIBPYTHON_EXTERN void (*PyErr_BadArgument)();
 LIBPYTHON_EXTERN PyObject* (*PyErr_Occurred)(void);
 LIBPYTHON_EXTERN void (*PyErr_NormalizeException)(PyObject**, PyObject**, PyObject**);
 LIBPYTHON_EXTERN int (*PyErr_GivenExceptionMatches)(PyObject *given, PyObject *exc);
@@ -330,7 +341,8 @@ LIBPYTHON_EXTERN int (*PyObject_IsTrue)(PyObject *o);
 LIBPYTHON_EXTERN PyObject* (*Py_CompileStringExFlags)(const char *str, const char *filename, int start, PyCompilerFlags *flags, int optimize);
 
 LIBPYTHON_EXTERN void* (*PyCapsule_Import)(const char *name, int no_block);
-  
+
+LIBPYTHON_EXTERN PyObject* (*PyObject_Type)(PyObject* o);
 #define PyObject_TypeCheck(o, tp) ((PyTypeObject*)Py_TYPE(o) == (tp)) || PyType_IsSubtype((PyTypeObject*)Py_TYPE(o), (tp))
 
 #define PyType_Check(o) PyObject_TypeCheck(o, PyType_Type)
@@ -708,6 +720,6 @@ LIBPYTHON_EXTERN PyThreadState* (*PyThreadState_Next)(PyThreadState*);
 /* End PyFrameObject */
 
 } // namespace libpython
+} // namespace reticulate
 
-#endif
-
+#endif /* RETICULATE_LIBPYTHON_H */
