@@ -168,3 +168,32 @@ test_that("NaT is converted to NA", {
   
 })
 
+test_that("pandas NAs are converted to R NAs", {
+  skip_if_no_pandas()
+  
+  code <- "
+import pandas as pd
+df = pd.DataFrame({'a': [1, 2, 3], 'b': [10, 20, pd.NA]})
+"
+  
+  locals <- py_run_string(code, local = TRUE, convert = TRUE)
+  
+  df <- locals$df
+  expect_true(is.na(df$b[[3]]))
+  
+  pd <- import("pandas", convert = FALSE)
+  pdNA <- py_to_r(py_get_attr(pd, "NA"))
+  expect_true(is.na(pdNA))
+  
+})
+
+test_that("categorical NAs are handled", {
+  skip_if_no_pandas()
+  
+  df <- data.frame(x = factor("a", NA))
+  pdf <- r_to_py(df)
+  rdf <- py_to_r(pdf)
+  attr(rdf, "pandas.index") <- NULL
+  expect_equal(df, rdf)
+  
+})
